<?php

namespace EventPortal\Domain\Agenda\Actions;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Agenda\SessionInput;
use EventPortal\Domain\Agenda\SessionOverlap;
use EventPortal\Domain\Form\Facades\Form;

class CreateSession
{
    public function execute(Pathway $pathway, array $input = []): Session
    {
        $input = SessionInput::from($input)
            ->forceDate($pathway->agenda->date)
            ->formatForDatabase();

        $form = Form::make('session.create')
            ->model(new Session)
            ->useAlternativeNames()
            ->context($pathway)
            ->validate($input);

        $overlaps = Session::query()
            ->forPathway($pathway)
            ->completelyOverlaps($input['starting_at'], $input['ending_at'])
            ->exists();

        SessionOverlap::throwValidationExceptionIfOverlapping($form->getValidator(), $overlaps);

        return $form->save($input)->getModel();
    }
}
