<?php

namespace EventPortal\Domain\Table;

use Closure;
use EventPortal\Domain\Component\Column\ActionColumn;
use EventPortal\Domain\Component\Column\Column;
use EventPortal\Domain\Component\Value;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection;

class TableRow
{
    /** @var array */
    public $action = [];

    private $model;

    private $context;

    public function __construct(Model $model, Table $table, Collection $columns)
    {
        $this->model = $model;
        $this->context = $table->context;
        $this->action = $table->action;

        $this->setFields();
        $this->setFieldsThatAreClosures($columns);
        $this->setActions();
    }

    private function setFields(): void
    {
        foreach ($this->model->getAttributes() as $attributeKey => $value) {
            $key = $attributeKey;

            if ($attributeKey === 'status') {
                $key = 'readable_status';
            }

            $this->$attributeKey = Value::get($this->model->$key);
        }
    }

    private function setFieldsThatAreClosures(Collection $columns): void
    {
        $columns
            ->reject(function ($column) {
                return $column instanceof ActionColumn;
            })
            ->filter(function (Column $column) {
                return $column->value !== null;
            })
            ->each(function (Column $column): void {
                $key = $column->name;

                if ($column->value instanceof Closure) {
                    $this->$key = app()->call(
                        $column->value,
                        $this->getParameters()
                    );

                    return;
                }

                $this->$key = $column->value;
            });
    }

    private function setActions(): void
    {
        if ($this->action === null) {
            return;
        }

        $this->action = app()->call($this->action, $this->getParameters());
    }

    private function getParameters(): array
    {
        $models = [$this->model::KEY => $this->model];

        if ($this->context !== null) {
            $models = array_merge($this->context, $models);
        }

        return $models;
    }
}
